/*
 * Simulator of MCS51 (sim.cc)
 *
 * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_GETOPT_H
# include <getopt.h>
#endif
#include <ctype.h>

#include "i_string.h"

#include "simcl.h"


/*
 * Program arguments
 */

cl_prg_arg::cl_prg_arg(char sn, char *ln, long long lv, char *sv):
  cl_base()
{
  short_name= sn;
  long_name = ln?strdup(ln):0;
  l_value= lv;
  s_value= sv?strdup(sv):0;
}

cl_prg_arg::~cl_prg_arg(void)
{
  if (long_name)
    free(long_name);
  if (s_value)
    free(s_value);
}

long long
cl_prg_arg::get_lvalue(void)
{
  return(l_value);
}

char *
cl_prg_arg::get_svalue(void)
{
  return(s_value);
}


/*
 * Simulator
 */

cl_sim::cl_sim(char *more_args, int iargc, char *iargv[]):
  cl_base()
{
  argc= iargc; argv= iargv;
  uc= 0;
  cmd= 0;
  arguments= new cl_list(2, 2);
  accept_args= more_args?strdup(more_args):0;
}

/*cl_sim::cl_sim(class cl_uc *auc):
  cl_base()
{
  uc= auc;
}*/

int
cl_sim::init(void)
{
  cl_base::init();
  proc_arguments(argc, argv);
  if (!(uc= mk_controller()))
    return(1);
  uc->init();
  if (!(cmd= mk_commander(get_sarg('c'), get_sarg('c'))))
    return(2);
  cmd->init();
  return(0);
}

cl_sim::~cl_sim(void)
{
  if (uc)
    delete uc;
}

int
cl_sim::proc_arguments(int argc, char *argv[])
{
  int c;
  char *opts;

  opts= (char*)malloc((accept_args?strlen(accept_args):0)+100);
  strcpy(opts, "vV");
  if (accept_args)
    strcat(opts, accept_args);
  opterr= 0;
  while((c= getopt(argc, argv, opts)) != -1)
    switch (c)
      {
      case 'v':
	printf("%s: %s\n", argv[0], VERSIONSTR);
        exit(0);
        break;
      case 'V':
	break;
      case '?':
	if (c= proc_arg(c, optarg))
	  exit(c);
	/*if (isprint(optopt))
          fprintf(stderr, "Unknown option `-%c'.\n", optopt);
        else
          fprintf(stderr, "Unknown option character `\\x%x'.\n", optopt);
	  exit(1);*/
        break;
      default:
        if (c= proc_arg(c, optarg))
	  exit(c);
      }
  
}

int
cl_sim::proc_arg(char arg, char *optarg)
{}

long long
cl_sim::get_larg(char name)
{
  class cl_prg_arg *a;
  int i;

  for (i= 0; i < arguments->count; i++)
    {
      a= (class cl_prg_arg *)(arguments->at(i));
      if (a->short_name == name)
	return(a->get_lvalue());
    }
  return(0);
}

char *
cl_sim::get_sarg(char name)
{
  class cl_prg_arg *a;
  int i;

  for (i= 0; i < arguments->count; i++)
    {
      a= (class cl_prg_arg *)(arguments->at(i));
      if (a->short_name == name)
	return(a->get_svalue());
    }
  return(0);
}


class cl_cmd *
cl_sim::mk_commander(char *con_in, char *con_out)
{
  return(new cl_cmd(con_in, con_out));
}

class cl_uc *
cl_sim::mk_controller(void)
{
  return(new cl_uc(this));
}


int
cl_sim::do_cmd(void)
{
  //  read_command(cmd_in, cmd_out);
}


/* End of sim.cc */
